# eblib.py
# Tamito KAJIYAMA <13 September 2001>
# $Id: eblib.py,v 1.1 2001/09/18 09:06:10 kajiyama Exp $

from eb import *

import string

class EB_Hit:
    def __init__(self, book, subbook, position):
        self.book = book # EB instance
        self.subbook = subbook
        self.position = position # (heading, text)
    def heading(self):
        return self.book.get_content(
            self.subbook, self.position[0], eb_heading)
    def text(self):
        return self.book.get_content(
            self.subbook, self.position[1], eb_text)

class EB:
    def __init__(self, dir):
        self.book     = EB_Book()
        self.appendix = EB_Appendix()
        self.hookset  = EB_Hookset()
        eb_set_hooks(self.hookset, (
            (EB_HOOK_NEWLINE,           self.handle_newline),
            (EB_HOOK_NARROW_FONT,       self.handle_font),
            (EB_HOOK_WIDE_FONT,         self.handle_font),
            (EB_HOOK_BEGIN_NARROW,      self.handle_tags),
            (EB_HOOK_END_NARROW,        self.handle_tags),
            (EB_HOOK_BEGIN_SUBSCRIPT,   self.handle_tags),
            (EB_HOOK_END_SUBSCRIPT,     self.handle_tags),
            (EB_HOOK_BEGIN_SUPERSCRIPT, self.handle_tags),
            (EB_HOOK_END_SUPERSCRIPT,   self.handle_tags),
            (EB_HOOK_BEGIN_TABLE,       self.handle_tags),
            (EB_HOOK_END_TABLE,         self.handle_tags),
            (EB_HOOK_BEGIN_NO_NEWLINE,  self.handle_tags),
            (EB_HOOK_END_NO_NEWLINE,    self.handle_tags),
            (EB_HOOK_BEGIN_EMPHASIS,    self.handle_tags),
            (EB_HOOK_END_EMPHASIS,      self.handle_tags),
            (EB_HOOK_BEGIN_PICTURE,     self.handle_tags),
            (EB_HOOK_END_PICTURE,       self.handle_tags),
            (EB_HOOK_BEGIN_MENU,        self.handle_tags),
            (EB_HOOK_END_MENU,          self.handle_tags),
            (EB_HOOK_BEGIN_SOUND,       self.handle_tags),
            (EB_HOOK_END_SOUND,         self.handle_tags),
            (EB_HOOK_BEGIN_REFERENCE,   self.handle_tags),
            (EB_HOOK_END_REFERENCE,     self.handle_tags),
            (EB_HOOK_BEGIN_KEYWORD,     self.handle_tags),
            (EB_HOOK_END_KEYWORD,       self.handle_tags),
#           (EB_HOOK_SET_INDENT,        None),
#           (EB_HOOK_ISO8859_1,         None),
#           (EB_HOOK_NARROW_JISX0208,   None),
#           (EB_HOOK_WIDE_JISX0208,     None),
#           (EB_HOOK_INITIALIZE,        None),
#           (EB_HOOK_GB2312,            None),
            (EB_HOOK_STOPCODE,          self.handle_stopcode)))
        self.bind(dir)
        self.set_subbook(0)
    def handle_newline(self, book, appendix, buffer, code, argv):
        buffer.set(self.hook_newline())
        return 0
    def handle_font(self, book, appendix, buffer, code, argv):
        ##print "handle_font: code=%d, argv=%s" % (code, repr(argv))
        if code == EB_HOOK_NARROW_FONT:
            if eb_narrow_alt_character_text(appendix, argv[0], buffer) < 0:
                buffer.set(self.hook_narrow_font(argv[0]))
        elif code == EB_HOOK_WIDE_FONT:
            if eb_wide_alt_character_text(appendix, argv[0], buffer) < 0:
                buffer.set(self.hook_wide_font(argv[0]))
        return 0
    def handle_tags(self, book, appendix, buffer, code, argv):
        ##print "handle_tags: code=%d, argv=%s" % (code, repr(argv))
        if code == EB_HOOK_BEGIN_NARROW:
            buffer.set(self.hook_begin_narrow(argv))
        elif code == EB_HOOK_END_NARROW:
            buffer.set(self.hook_end_narrow(argv))
        elif code == EB_HOOK_BEGIN_SUBSCRIPT:
            buffer.set(self.hook_begin_subscript(argv))
        elif code == EB_HOOK_END_SUBSCRIPT:
            buffer.set(self.hook_end_subscript(argv))
        elif code == EB_HOOK_BEGIN_SUPERSCRIPT:
            buffer.set(self.hook_begin_superscript(argv))
        elif code == EB_HOOK_END_SUPERSCRIPT:
            buffer.set(self.hook_end_superscript(argv))
        elif code == EB_HOOK_BEGIN_TABLE:
            buffer.set(self.hook_begin_table(argv))
        elif code == EB_HOOK_END_TABLE:
            buffer.set(self.hook_end_table(argv))
        elif code == EB_HOOK_BEGIN_NO_NEWLINE:
            buffer.set(self.hook_begin_no_newline(argv))
        elif code == EB_HOOK_END_NO_NEWLINE:
            buffer.set(self.hook_end_no_newline(argv))
        elif code == EB_HOOK_BEGIN_EMPHASIS:
            buffer.set(self.hook_begin_emphasis(argv))
        elif code == EB_HOOK_END_EMPHASIS:
            buffer.set(self.hook_end_emphasis(argv))
        elif code == EB_HOOK_BEGIN_PICTURE:
            buffer.set(self.hook_begin_picture(argv))
        elif code == EB_HOOK_END_PICTURE:
            buffer.set(self.hook_end_picture(argv))
        elif code == EB_HOOK_BEGIN_MENU:
            buffer.set(self.hook_begin_menu(argv))
        elif code == EB_HOOK_END_MENU:
            buffer.set(self.hook_end_menu(argv))
        elif code == EB_HOOK_BEGIN_SOUND:
            buffer.set(self.hook_begin_sound(argv))
        elif code == EB_HOOK_END_SOUND:
            buffer.set(self.hook_end_sound(argv))
        elif code == EB_HOOK_BEGIN_REFERENCE:
            buffer.set(self.hook_begin_reference(argv))
        elif code == EB_HOOK_END_REFERENCE:
            buffer.set(self.hook_end_reference(argv))
        elif code == EB_HOOK_BEGIN_KEYWORD:
            buffer.set(self.hook_begin_keyword(argv))
        elif code == EB_HOOK_END_KEYWORD:
            buffer.set(self.hook_end_keyword(argv))
        return 0
    def handle_stopcode(self, book, appendix, buffer, code, argv):
        ##print "handle_stopcode: code=%d, argv=%s" % (code, repr(argv))
        return eb_hook_stopcode_mixed(book, appendix, buffer, code, argv)
    def get_content(self, subbook, position, func):
        # save current subbook
        current_subbook = self.subbook()
        # get content at the specified subbook/position
        if subbook is not None:
            self.set_subbook(subbook)
        eb_seek(self.book, position)
        buffer = []
        while 1:
            data = func(self.book, self.appendix, self.hookset)
            if not data:
                break
            buffer.append(data)
        # restore current subbook
        self.set_subbook(current_subbook)
        return string.join(buffer, '')
    # callbacks
    def hook_newline(self):
        return "\n"
    def hook_narrow_font(self, code):
        return "?"
    def hook_wide_font(self, code):
        return "?"
    def hook_begin_narrow(self, argv):
        return ""
    def hook_end_narrow(self, argv):
        return ""
    def hook_begin_subscript(self, argv):
        return ""
    def hook_end_subscript(self, argv):
        return ""
    def hook_begin_superscript(self, argv):
        return ""
    def hook_end_superscript(self, argv):
        return ""
    def hook_begin_table(self, argv):
        return ""
    def hook_end_table(self, argv):
        return ""
    def hook_begin_no_newline(self, argv):
        return ""
    def hook_end_no_newline(self, argv):
        return ""
    def hook_begin_emphasis(self, argv):
        return ""
    def hook_end_emphasis(self, argv):
        return ""
    def hook_begin_picture(self, argv):
        return ""
    def hook_end_picture(self, argv):
        return ""
    def hook_begin_menu(self, argv):
        return ""
    def hook_end_menu(self, argv):
        return ""
    def hook_begin_sound(self, argv):
        return ""
    def hook_end_sound(self, argv):
        return ""
    def hook_begin_reference(self, argv):
        return ""
    def hook_end_reference(self, argv):
        return ""
    def hook_begin_keyword(self, argv):
        return ""
    def hook_end_keyword(self, argv):
        return ""
    # functions
    def bind(self, dir):
        return eb_bind(self.book, dir)
    def suspend(self):
        return eb_suspend(self.book)
    def clear(self):
        return eb_clear(self.book)
    def is_bound(self):
        return eb_is_bound(self.book)
    def path(self):
        return eb_path(self.book)
    def character_code(self):
        return eb_character_code(self.book)
    def disc_type(self):
        return eb_disc_type(self.book)
    def initialize_all_subbooks(self):
        return eb_initialize_all_subbooks(self.book)
    def subbook_list(self):
        return eb_subbook_list(self.book)
    def set_subbook(self, subbook):
        return eb_set_subbook(self.book, subbook)
    def unset_subbook(self):
        return eb_unset_subbook(self.book)
    def subbook(self):
        return eb_subbook(self.book)
    def subbook_title(self):
        return eb_subbook_title(self.book)
    def subbook_directory(self):
        return eb_subbook_directory(self.book)
    def subbook_title2(self, subbook):
        return eb_subbook_title2(self.book, subbook)
    def subbook_directory2(self, subbook):
        return eb_subbook_directory2(self.book, subbook)
    def copyright(self):
        if eb_have_copyright(self.book):
            return EB_Hit(self, None, eb_copyright(self.book))
        return None
    def menu(self):
        if eb_have_menu(self.book):
            return EB_Hit(self, None, eb_menu(self.book))
        return None
    def search_exactword(self, word):
        return self.do_search(eb_search_exactword, word)
    def search_word(self, word):
        return self.do_search(eb_search_word, word)
    def search_endword(self, word):
        return self.do_search(eb_search_endword, word)
    def do_search(self, func, word):
        func(self.book, word)
        buffer = []
        while 1:
            hitlist = eb_hit_list(self.book)
            if hitlist == -1:
                raise
            if not hitlist:
                break
            subbook = self.subbook()
            for hit in hitlist:
                buffer.append(EB_Hit(self, subbook, hit))
        return buffer

def test():
    import sys
    argc = len(sys.argv)
    if argc != 2 and argc != 4:
        sys.stderr.write("Usage: python eblib.py dir [subbook] [word]\n")
        sys.exit(1)

    class EBTest(EB):
        def hook_narrow_font(self, code):
            return "<gaiji=h%04x>" % code
        def hook_wide_font(self, code):
            return "<gaiji=z%04x>" % code
        def hook_begin_reference(self, argv):
            return "<reference>"
        def hook_end_reference(self, argv):
            return "</reference=%x:%x>" % (argv[1], argv[2])
        def hook_begin_keyword(self, argv):
            return "<keyword>"
        def hook_end_keyword(self, argv):
            return "</keyword>"

    eb = EBTest(sys.argv[1])
    if argc == 2:
        for subbook in eb.subbook_list():
            print "#%d. %s (%s)" % (subbook,
                                    eb.subbook_title2(subbook),
                                    eb.subbook_directory2(subbook))
    else:
        eb.set_subbook(int(sys.argv[2]))
        word = sys.argv[3]
        if word[-1:] == "*":
            func = eb.search_word
            word = word[:-1]
        elif word[:1] == "*":
            func = eb.search_endword
            word = word[1:]
        else:
            func = eb.search_exactword
        for hit in func(word):
            print "-" * 40
            print hit.heading()
            print hit.text()

if __name__ == "__main__":
    test()
